import EventTarget$1 from '@ungap/event-target';
import { CustomEvent as CustomEvent$1 } from '@web-media/event-target';

/**
 * Represents the state of an OpenPromise instance.
 */
declare enum OpenPromiseState {
    Idle = "idle",
    Pending = "pending",
    Fulfilled = "fulfilled",
    Rejected = "rejected"
}
/**
 * An implementation of a Promise that allows the executor function to be
 * executed lazily and provides access to the resolve and reject functions.
 *
 * @typeparam T The type of the resolved value of the promise.
 */
declare class PromiseResolvedError extends Error {
    name: string;
    constructor();
}
declare class PromiseExecutedError extends Error {
    name: string;
    constructor();
}
/**
 * A type that defines the `resolve` function for a Promise instance.
 */
type Resolve<T> = (value: T | PromiseLike<T>) => void;
/**
 * A type that defines the `reject` function for a Promise instance.
 */
type Reject = (reason: Error) => void;
/**
 * A type that defines the executor function for a Promise instance.
 */
type PromiseExecutor<T> = (resolve: Resolve<T>, reject: Reject) => void;
declare class OpenPromise<T> implements Promise<T> {
    readonly executor: PromiseExecutor<T> | null;
    readonly lazy: boolean;
    private _resolve;
    private _reject;
    resolvedValue: T | null;
    /**
     * The rejected reason of the promise. Set to null until the promise has been
     * rejected.
     */
    rejectedReason: Error | null;
    /**
     * The current state of the promise.
     */
    state: OpenPromiseState;
    /**
     * The underlying promise instance.
     */
    readonly promise: Promise<T>;
    /**
     * Attaches callbacks for the resolution and/or rejection of the Promise.
     *
     * @param onFulfilled The callback to execute when the Promise is resolved.
     * @param onRejected The callback to execute when the Promise is rejected.
     * @returns A Promise for the completion of which ever callback is executed.
     */
    readonly then: Promise<T>['then'];
    /**
     * Attaches a callback for only the rejection of the Promise.
     *
     * @param onRejected The callback to execute when the Promise is rejected.
     * @returns A Promise for the completion of the callback.
     */
    readonly catch: Promise<T>['catch'];
    /**
     * Attaches a callback for the resolution and/or rejection of the Promise.
     * The callback is executed when the Promise is settled, whether fulfilled or
     * rejected.
     *
     * @param onFinally The callback to execute when the Promise is settled.
     * @returns A Promise for the completion of the callback.
     */
    readonly finally: Promise<T>['finally'];
    /**
     * A string representing the object type. In this case, it is set to
     * `'OpenPromise'`.
     */
    readonly [Symbol.toStringTag]: string;
    /**
     * Creates a new OpenPromise instance.
     *
     * @param executor An optional function to be executed immediately upon
     *   construction.
     * @param lazy Whether or not to delay execution until the `execute()` method
     *   is called.
     */
    constructor(executor?: PromiseExecutor<T> | null, lazy?: boolean);
    /**
     * Executes the Promise if it has not already been executed.
     */
    execute: () => void;
    /**
     * Resolves the Promise with the given value.
     *
     * @param value The value with which to resolve the Promise.
     */
    resolve: (value: T | PromiseLike<T>) => void;
    /**
     * A function that is used to reject the promise with a reason.
     *
     * @param reason The reason why the promise was rejected.
     */
    reject: (reason: Error) => void;
}

/**
 * A queue that processes tasks asynchronously in frames.
 */
declare class FrameFillingQueue extends EventTarget$1 {
    readonly concurrent: number;
    readonly dependencyQueue?: Queue | undefined;
    readonly queueId: string;
    readonly protectedTime: number;
    /**
     * The tasks currently in the queue.
     */
    private queue;
    /**
     * A map of tasks to their unique identifiers.
     */
    readonly taskMap: Map<QueuedTask, string>;
    /**
     * The number of cleared tasks.
     */
    clearedTasks: number;
    /**
     * The number of rejected promises.
     */
    rejected: number;
    /**
     * The number of resolved promises.
     */
    resolved: number;
    /**
     * The number of tasks currently being processed.
     */
    working: number;
    /**
     * Returns the number of remaining tasks in the queue.
     */
    get remainTasks(): number;
    /**
     * Creates a new instance of `FrameFillingQueue`.
     *
     * @param concurrent - The maximum number of tasks to execute concurrently.
     * @param dependencyQueue - An optional dependency queue that must be empty
     *   before tasks in this queue can be executed.
     * @param queueId - An optional unique identifier for the queue.
     * @param protectedTime - The amount of time (in milliseconds) that must
     *   remain in the current frame before the next task can be executed.
     */
    constructor(concurrent?: number, dependencyQueue?: Queue | undefined, queueId?: string, protectedTime?: number);
    /**
     * The number of tasks in the queue.
     */
    get length(): number;
    /**
     * Adds a task to the queue.
     *
     * @param task - The task to add to the queue.
     * @param taskId - An optional unique identifier for the task.
     */
    add: (task: QueuedTask, taskId?: string) => void;
    /**
     * Removes a task from the queue.
     *
     * @param task - The task to remove from the queue.
     * @param silent - An optional flag to indicate whether to suppress the
     *   `QueueUpdateEvent`.
     */
    remove(task: QueuedTask, silent?: boolean): void;
    /**
     * Executes a task and returns a promise that resolves when the task is
     * complete.
     *
     * @param task - The task to execute.
     */
    private executeTask;
    /**
     * Handles a promise in the queue, updating the relevant counters and
     * triggering the execution of the next task if the concurrency limit allows
     * it.
     *
     * @param p - The promise to handle.
     * @param results - An array to which the promise will be pushed.
     * @returns A promise that resolves when the input promise has settled.
     */
    private handlePromise;
    /**
     * Executes the next available task in the queue. If the queue is not empty
     * and the number of working tasks is less than `concurrent`, a task will be
     * removed from the queue and executed.
     *
     * @returns A promise that resolves when all tasks have completed.
     */
    tick: () => Promise<({
        status: string;
        value: any;
    } | {
        status: string;
        reason: any;
    })[]>;
}

/**
 * SequentialQueue is a queue of tasks that executes tasks in sequence up to a
 * certain concurrency limit.
 * Tasks can be either a synchronous function or an OpenPromise (a lazy promise
 * that will only execute when executed).
 *
 * @typeparam T The type of the result produced by the OpenPromise task.
 */
declare class SequentialQueue extends EventTarget$1 {
    readonly concurrency: number;
    readonly dependencyQueue?: Queue | undefined;
    readonly queueId: string;
    /**
     * The queue of tasks to execute.
     */
    protected queue: Array<QueuedTask>;
    /**
     * The number of tasks that have been cleared from the queue.
     */
    protected internalClearedTasks: number;
    /**
     * A map of QueuedTask objects to their associated task IDs.
     */
    readonly taskMap: Map<QueuedTask, string>;
    /**
     * Creates a new SequentialQueue object.
     *
     * @param concurrency The number of tasks to execute concurrently.
     *   Defaults to 1.
     * @param dependencyQueue A queue of tasks that must complete before tasks in
     *   this queue can be executed. Defaults to undefined.
     * @param queueId An optional ID for this queue. If not specified, a random ID
     *   will be generated.
     */
    constructor(concurrency?: number, dependencyQueue?: Queue | undefined, queueId?: string);
    /**
     * Returns the number of tasks remaining in the queue.
     */
    get remainTasks(): number;
    /**
     * Returns the number of tasks in the queue.
     */
    get length(): number;
    /**
     * Returns the number of tasks that have been cleared from the queue.
     */
    get clearedTasks(): number;
    /**
     * Adds a task to the queue.
     *
     * @param task The task to add to the queue.
     * @param taskId An optional ID for the task. If specified, the task will be
     * added to the taskMap with this ID.
     */
    add(task: QueuedTask, taskId?: string): void;
    /**
     * Removes a task from the queue.
     *
     * @param task The task to remove from the queue.
     */
    remove(task: QueuedTask): void;
    /**
     * Executes the next set of tasks in the queue, up to the concurrency limit.
     * If a dependencyQueue is provided and has remaining tasks, the method
     * returns immediately without executing any tasks.
     * Returns a Promise that resolves with an array of results from any
     * OpenPromise tasks that were executed during this tick.
     */
    tick(): Promise<({
        status: string;
        value: any;
    } | {
        status: string;
        reason: any;
    })[]>;
}

/**
 * The `QueueType` enum represents the two types of queues that can be used in
 * the `TimeSlicingQueue` class:
 * a sequential queue or a frame-filling queue.
 */
declare enum QueueType {
    Sequential = "sequential",
    FrameFilling = "frame-filling"
}
/**
 * The `TimeSlicingQueue` class is a task queue that schedules tasks to run
 * at a later time to avoid blocking the main thread.
 *
 * @template T The type of the queued tasks.
 */
declare class TimeSlicingQueue extends EventTarget {
    readonly concurrency: number;
    readonly type: QueueType;
    readonly protectedTime: number;
    readonly dependencyQueue?: Queue | undefined;
    readonly queueId: string;
    /**
     * A flag indicating whether the queue is currently running.
     */
    running: boolean;
    /**
     * Indicates whether a tick has been scheduled.
     *
     * @private
     */
    private tickScheduled;
    /**
     * The time of the last successful tick.
     *
     * @private
     */
    private lastSuccessfulTickTime;
    /**
     * The queue used to store tasks.
     *
     * @private
     */
    private queue;
    /**
     * Returns a map of the tasks in the queue and their corresponding IDs.
     */
    get taskMap(): Map<QueuedTask, string>;
    /**
     * Returns the number of tasks in the queue.
     */
    get length(): number;
    /**
     * Returns the number of tasks in the queue.
     *
     * This property is identical to `length` and is provided for
     * compatibility with the `Queue` interface.
     */
    get remainTasks(): number;
    /**
     * Constructs a new `TimeSlicingQueue` object.
     *
     * @param concurrency The maximum number of tasks that can be run
     *   concurrently.
     * @param type The type of the queue to use: either a sequential queue or a
     *   frame-filling queue.
     * @param protectedTime The minimum amount of time (in milliseconds) between
     *   ticks to ensure that the main thread is not blocked.
     * @param dependencyQueue A dependency queue to use.
     * @param queueId An optional ID to use for the queue.
     */
    constructor(concurrency?: number, type?: QueueType, protectedTime?: number, dependencyQueue?: Queue | undefined, queueId?: string);
    /**
     * Runs the queue.
     */
    run: () => void;
    /**
     * Adds a task to the queue.
     * @param task The task to add.
     * @param taskId An optional ID to assign to the task.
     */
    add: (task: QueuedTask, taskId?: string) => void;
    get remove(): ((task: QueuedTask, silent?: boolean | undefined) => void) | ((task: QueuedTask) => void);
    /**
     * Logs the tasks that are still in the queue.
     *
     * @private
     * @param title The title of the log.
     */
    private logRemainedTask;
    /**
     * Schedules a tick to process the next task in the queue.
     *
     * @private
     */
    private scheduleTick;
    /**
     * Processes the next task in the queue.
     *
     * @private
     */
    private tickOnce;
    /**
     * Stops the queue.
     */
    stop: () => void;
}

type QueuedTask = (() => void) | OpenPromise<any>;
type Queue = SequentialQueue | FrameFillingQueue | TimeSlicingQueue;

/**
 * Converts an array of Promises to an array of objects that represent the
 * settled state of each Promise. A Promise is considered settled if it has
 * either fulfilled or rejected.
 *
 * @param promises - An array of Promises to be settled.
 * @returns A Promise that resolves to an array of objects containing a
 * `status` property that is either "fulfilled" or "rejected", and a
 * corresponding property (`value` for fulfilled Promises, `reason` for rejected
 * Promises).
 */
declare const allSettled: (promises: (Promise<any> | OpenPromise<any>)[]) => Promise<({
    status: string;
    value: any;
} | {
    status: string;
    reason: any;
})[]>;

/**
 * Represents an event with additional data of type `T`.
 *
 * @typeparam T - The type of the additional data attached to the event.
 */
declare class CustomEvent<T> extends Event {
    /**
     * Additional data attached to the event.
     */
    detail: T | undefined;
    /**
     * Creates a new instance of `CustomEvent`.
     *
     * @param type - The type of the event.
     * @param options - An object containing options to customize the event.
     *   The `detail` property of the `options` object can be used to attach
     *   additional data of type `T` to the event.
     */
    constructor(type: string, options: EventInit & {
        detail: T;
    });
}

/**
 * Returns the current time in milliseconds, using `performance.now()` if
 * available, otherwise using `Date.now()`.
 * @returns The current time in milliseconds.
 */
declare const now: () => number;
/**
 * The target frame period, in milliseconds.
 */
declare const STABLE_FRAME_PERIOD = 17;
/**
 * The default time to use as the deadline when calculating
 * `IdleDeadline.timeRemaining()`, in milliseconds.
 */
declare const IDLE_TIME_REMAINING = 14;
/**
 * Returns the current average frame period, in milliseconds.
 * @returns The current average frame period, in milliseconds.
 */
declare const getFramePeriod: () => number;
/**
 * Initializes the `framePeriodBuffer` and starts an animation frame loop to
 * continuously update the buffer.
 */
declare const initializeTimeRemaining: () => void;
/**
 * Calculates the time remaining until the deadline, either using the
 * `IdleDeadline.timeRemaining()` method if available,
 * or by estimating the time remaining based on the current time and the
 * previous frame time.
 * @param deadline The deadline to calculate the time remaining for, in
 * milliseconds. Defaults to `IDLE_TIME_REMAINING`.
 * @returns The time remaining until the deadline, in milliseconds.
 */
declare const timeRemaining: (deadline?: number) => any;

/**
 * An enum representing different update actions that can be taken on a queue.
 */
declare enum QueueUpdateAction {
    /**
     * New task was added to the queue.
     */
    Add = "add",
    /**
     * Some task was removed from the queue.
     */
    Remove = "remove",
    /**
     * Some tasks was executed and removed from the queue.
     */
    Clear = "clear",
    /**
     * All tasks was executed.
     */
    Drain = "drain"
}

/**
 * Interface representing the details of a queue update event.
 */
interface IQueueUpdateEventDetail {
    /**
     * The number of tasks remaining in the queue.
     */
    remainTasks: number;
    /**
     * The number of tasks that were cleared from the queue.
     */
    clearedTasks: number;
    /**
     * The update action that was taken on the queue.
     */
    action: QueueUpdateAction;
}
/**
 * A custom event for updates to a queue.
 */
declare class QueueUpdateEvent extends CustomEvent$1<IQueueUpdateEventDetail> {
    /**
     * Creates a new `QueueUpdateEvent` instance.
     * @param queue An object representing the state of the queue.
     * @param action The update action that was taken on the queue.
     */
    constructor(queue: {
        remainTasks: number;
        clearedTasks: number;
    }, action: QueueUpdateAction);
}

/**
 * The `requestIdleCallback` function to use, either the native implementation
 * or the fallback implementation if the native implementation is not
 * supported by the browser.
 */
declare const requestIdleCallback: typeof globalThis.requestIdleCallback;
/**
 * Cancels a scheduled idle callback previously registered with
 * `requestIdleCallback()`.
 */
declare const cancelIdleCallback: typeof globalThis.cancelIdleCallback;

/**
 * Custom error class for when an OpenPromise is not lazy.
 */
declare class NotLazyOpenPromiseError extends Error {
    /**
     * The name of the error class.
     */
    name: string;
    /**
     * Creates a new instance of the error with a custom message.
     */
    constructor();
}

export { CustomEvent, FrameFillingQueue, IDLE_TIME_REMAINING, NotLazyOpenPromiseError, OpenPromise, OpenPromiseState, PromiseExecutedError, PromiseResolvedError, Queue, QueueType, QueueUpdateAction, QueueUpdateEvent, QueuedTask, STABLE_FRAME_PERIOD, SequentialQueue, TimeSlicingQueue, allSettled, cancelIdleCallback, getFramePeriod, initializeTimeRemaining, now, requestIdleCallback, timeRemaining };
//# sourceMappingURL=types.d.ts.map
